# This program source code file is part of KiCad, a free EDA CAD application.
# Copyright The KiCad Developers, see AUTHORS.TXT for contributors.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, you may find one here:
# https://www.gnu.org/licenses/gpl-3.0.en.html
# or you may search the http://www.gnu.org website for the version 32 license,
# or you may write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

"""
KiCad HTTP Lib Test Server
==========================

This module implements a test HTTP server using FastAPI.  It is a simple way to spin up a local
API server to test the Kicad HTTP feature.

fastAPI gives you a test interface at the /docs endpoint to observe the responses.

Usage
-----
Run the server using:
    python http_lib_test_server.py

The server will start on http://0.0.0.0:8000

Mock Data
---------
The current implementation includes mock data for testing. 

- One resistor category
- One sample resistor part (RC0603FR-0710KL)
- It assumes the default Kicad libraries are setup in the symbol/footprint table.
  it uses the basic resistor symbol and an 0603 footprint

Dependencies
-----------
- FastAPI: Web framework
- Pydantic: Data validation
- uvicorn: ASGI server

For development and testing purposes only. Not intended for production use.
"""
 

from typing import List, Optional, Dict

import uvicorn
from fastapi import FastAPI, HTTPException
from pydantic import BaseModel

app = FastAPI()

class Category(BaseModel):
    id: str
    name: str
    description: Optional[str] = None

class PartsChooser(BaseModel):
    id: str
    name: str
    description: Optional[str] = None

class PartField(BaseModel):
    value: str
    visible: Optional[str] = "true"

class DetailedPart(BaseModel):
    id: str
    name: Optional[str] = None
    symbolIdStr: str
    exclude_from_bom: str
    exclude_from_board: str
    exclude_from_sim: str
    fields: Dict[str, PartField]

categories = [
    Category(id="1", name="Resistors", description = "Resistors"),
]

parts_by_category: Dict[str, List[PartsChooser]] = {
    "1": [
        PartsChooser(
            id="1",
            name="RC0603FR-0710KL",
            description="10 kOhms ±1% 0.1W, 1/10W Chip Resistor 0603 (1608 Metric) Moisture Resistant Thick Film"
        )
    ]
}

detailed_parts = {
    "1": DetailedPart(
        id="1",
        name="RC0603FR-0710KL",
        symbolIdStr = "Device:R",
        exclude_from_bom="False",
        exclude_from_board="False",
        exclude_from_sim="True",
        fields={
            "footprint": PartField(
                value="Resistor_SMD:R_0603_1608Metric",
                visible="False"
            ),
            "datasheet": PartField(
                value="www.kicad.org",
                visible="False"
            ),
            "value": PartField(
                value="10k"
            ),
            "reference": PartField(
                value="R"
            ),
            "description": PartField(
                value="10 kOhms ±1% 0.1W, 1/10W Chip Resistor 0603 (1608 Metric) Moisture Resistant Thick Film",
                visible="False"
            ),
            "keywords": PartField(
                value="RES passive smd",
                visible="False"
            )
        }
    )
}

@app.get("/v1/")
async def get_endpoints():
    return {
        "categories": "",
        "parts": ""
    }

@app.get("/v1/categories.json", response_model=List[Category])
async def get_categories():

    return categories

@app.get("/v1/parts/category/{category_id}.json", response_model=List[PartsChooser])
async def get_parts_by_category(category_id: str):

    if category_id not in parts_by_category:
        raise HTTPException(status_code=404, detail=f"Category {category_id} not found")

    return parts_by_category[category_id]

@app.get("/v1/parts/{part_id}.json", response_model=DetailedPart)
async def get_part_details(part_id: str):

    if part_id not in detailed_parts:
        raise HTTPException(status_code=404, detail=f"Part {part_id} not found")

    return detailed_parts[part_id]

if __name__ == "__main__":
    uvicorn.run(app, host="0.0.0.0", port=8000)





#
# This program source code file is part of KiCad, a free EDA CAD application.
#
# Copyright (C) 2023 Mark Roszko <mark.roszko@gmail.com>
# Copyright (C) 2023 KiCad Developers
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

import utils
import cairosvg
import re
from pathlib import Path
import pytest
from typing import List


@pytest.mark.parametrize("test_file,output_dir,compare_fn,cli_args",
                            [("cli/basic_test/basic_test.kicad_sch", "basic_test", "cli/basic_test/basic_test.svg", []),
                             ("cli/basic_test/basic_test.kicad_sch", "basic_test_nobg_bnw_nods", "cli/basic_test/basic_test_nobg_bnw_nods.svg", ["--no-background-color", "--exclude-drawing-sheet", "--black-and-white"])
                             ])
def test_sch_export_svg( kitest,
                         test_file: str,
                         output_dir: str,
                         compare_fn: str,
                         cli_args: List[str] ):
    input_file = kitest.get_data_file_path( test_file )

    output_path =  kitest.get_output_path( "cli/{}/".format( output_dir ) )

    command = [utils.kicad_cli(), "sch", "export", "svg"]
    command.extend( cli_args )
    command.append( "-o" )
    command.append( str( output_path ) )
    command.append( input_file )

    stdout, stderr, exitcode = utils.run_and_capture( command )

    assert exitcode == 0
    assert stderr == ''
    assert stdout is not None

    stdout_regex = re.search("Plotted to '(.+)'", stdout)
    assert stdout_regex

    # now try and manipulate the extracted path
    output_svg_path = Path( stdout_regex.group(1) )
    assert output_svg_path.exists()

    kitest.add_attachment( output_svg_path )

    png_converted_from_svg_path = output_svg_path.with_suffix( '.png' )

    cairosvg.svg2png( url=str( output_svg_path ), write_to=str( png_converted_from_svg_path ), dpi=1200 )

    compare_file_path = kitest.get_data_file_path( compare_fn )
    compare_stem = f"orig_{output_dir}"
    compare_png_converted_from_svg_path = output_svg_path.with_suffix( '.png' ).with_stem(compare_stem)
    cairosvg.svg2png( url=str( compare_file_path ), write_to=str( compare_png_converted_from_svg_path ), dpi=1200 )

    assert utils.images_are_equal( png_converted_from_svg_path, compare_png_converted_from_svg_path  )


@pytest.mark.parametrize("test_file,output_fn,line_skip_count,skip_compare,cli_args",
                            [("cli/basic_test/basic_test.kicad_sch", "basic_test.netlist.kicadsexpr", 5, True, []),
                             ("cli/basic_test/basic_test.kicad_sch", "basic_test.netlist.kicadsexpr", 5, True,["--format=kicadsexpr"]),
                             ("cli/basic_test/basic_test.kicad_sch", "basic_test.netlist.kicadxml", 6, True,["--format=kicadxml"]),
                             ("cli/basic_test/basic_test.kicad_sch", "basic_test.netlist.cadstar", 3, False, ["--format=cadstar"]),
                             ("cli/basic_test/basic_test.kicad_sch", "basic_test.netlist.orcadpcb2", 1, False, ["--format=orcadpcb2"])
                             ])
def test_sch_export_netlist( kitest,
                             test_file: str,
                             output_fn: str,
                             line_skip_count: int,
                             skip_compare: bool,
                             cli_args: List[str] ):
    input_file = kitest.get_data_file_path( test_file )
    compare_filepath = kitest.get_data_file_path( "cli/basic_test/{}".format( output_fn ) )

    output_filepath =  kitest.get_output_path( "cli/" ).joinpath( output_fn )

    command = [utils.kicad_cli(), "sch", "export", "netlist"]
    command.extend( cli_args )
    command.append( "-o" )
    command.append( str( output_filepath ) )
    command.append( input_file )

    stdout, stderr, exitcode = utils.run_and_capture( command )

    assert exitcode == 0
    assert stderr == ''

    # some of our netlist formats are not cross platform so skip for now
    if not skip_compare:
        assert utils.textdiff_files( compare_filepath, str( output_filepath ), line_skip_count )

    kitest.add_attachment( str( output_filepath ) )


@pytest.mark.parametrize("test_file,output_fn,cli_args",
                            [("cli/basic_test/basic_test.kicad_sch", "basic_test.pdf", []),
                             ("cli/basic_test/basic_test.kicad_sch", "basic_test.bnw.nods.nobg.pdf", ["--black-and-white","--exclude-drawing-sheet","--no-background-color"]),
                             ("cli/basic_test/basic_test.kicad_sch", "basic_test.pone.pdf", ["--pages", "1"])
                             ])
def test_sch_export_pdf( kitest,
                         test_file: str,
                         output_fn: str,
                         cli_args: List[str] ):
    input_file = kitest.get_data_file_path( test_file )

    output_filepath =  kitest.get_output_path( "cli/" ).joinpath( output_fn )

    command = [utils.kicad_cli(), "sch", "export", "pdf"]
    command.extend( cli_args )
    command.append( "-o" )
    command.append( str( output_filepath ) )
    command.append( input_file )

    stdout, stderr, exitcode = utils.run_and_capture( command )

    assert exitcode == 0
    assert stderr == ''

    kitest.add_attachment( str( output_filepath ) )


@pytest.mark.parametrize("test_file,output_fn,line_skip_count,cli_args",
                            [("cli/basic_test/basic_test.kicad_sch", "basic_test.pythonbom", 6, [])
                             ])
def test_sch_export_pythonbom( kitest,
                         test_file: str,
                         output_fn: str,
                         line_skip_count: int,
                         cli_args: List[str] ):
    input_file = kitest.get_data_file_path( test_file )
    compare_filepath = kitest.get_data_file_path( "cli/basic_test/{}".format( output_fn ) )

    output_filepath =  kitest.get_output_path( "cli/" ).joinpath( output_fn )

    command = [utils.kicad_cli(), "sch", "export", "python-bom"]
    command.extend( cli_args )
    command.append( "-o" )
    command.append( str( output_filepath ) )
    command.append( input_file )

    stdout, stderr, exitcode = utils.run_and_capture( command )

    assert exitcode == 0
    assert stderr == ''

    # pythonbom is not currently crossplatform (platform specific paths) to enable diffs

    kitest.add_attachment( str( output_filepath ) )



@pytest.mark.parametrize("test_file,output_fn,line_skip_count,skip_compare,expected_exit_code,cli_args",
                            [("cli/basic_test/basic_test.kicad_sch", "basic_test.erc.rpt", 1, False, 0, []),
                             ("cli/basic_test/basic_test.kicad_sch", "basic_test.erc.rpt", 1, False, 0, ["--format=report"]),
                             ("cli/basic_test/basic_test.kicad_sch", "basic_test.erc.json", 5, False,0, ["--format=json"]),
                             ("cli/basic_test/basic_test.kicad_sch", "basic_test.erc.unitsin.rpt", 1, False, 0, ["--format=report", "--units=in"]),
                             ])
def test_sch_export_erc( kitest,
                             test_file: str,
                             output_fn: str,
                             line_skip_count: int,
                             skip_compare: bool,
                             expected_exit_code: int,
                             cli_args: List[str] ):
    input_file = kitest.get_data_file_path( test_file )
    compare_filepath = kitest.get_data_file_path( "cli/basic_test/{}".format( output_fn ) )

    output_filepath =  kitest.get_output_path( "cli/" ).joinpath( output_fn )

    command = [utils.kicad_cli(), "sch", "erc"]
    command.extend( cli_args )
    command.append( "-o" )
    command.append( str( output_filepath ) )
    command.append( input_file )

    stdout, stderr, exitcode = utils.run_and_capture( command )

    assert exitcode == expected_exit_code
    assert stderr == ''

    # some of our netlist formats are not cross platform so skip for now
    if not skip_compare:
        assert utils.textdiff_files( compare_filepath, str( output_filepath ), line_skip_count )

    kitest.add_attachment( str( output_filepath ) )



import pytest
import os
import pcbnew
import tempfile


from pcbnew import *


BACK_COPPER = 'Back_Copper'
B_CU = 'B.Cu'
NEW_NAME = 'My_Fancy_Layer_Name'


class TestBoardClass:
    pcb : pcbnew.BOARD = None

    def setup_method(self):
        self.pcb = LoadBoard("../data/pcbnew/complex_hierarchy.kicad_pcb")
        self.TITLE="Test Board"
        self.COMMENT1="For load/save test"
        self.FILENAME=tempfile.mktemp()+".kicad_pcb"

    def test_pcb_find_module(self):
        module = self.pcb.FindFootprintByReference('P1')
        assert module.GetReference() =='P1'

    def test_pcb_get_track_count(self):
        pcb = BOARD()

        assert pcb.Tracks().size() == 0

        track0 = PCB_TRACK(pcb)
        pcb.Add(track0)
        assert pcb.Tracks().size() == 1

        track1 = PCB_TRACK(pcb)
        pcb.Add(track1)
        assert pcb.Tracks().size() == 2

    def test_pcb_bounding_box(self):
        pcb = BOARD()
        track = PCB_TRACK(pcb)
        pcb.Add(track)

        track.SetStart(VECTOR2I_MM(10.0, 10.0))
        track.SetEnd(VECTOR2I_MM(20.0, 30.0))

        track.SetWidth(FromMM(0.5))

        #!!! THIS FAILS? == 0.0 x 0.0 ??
        #height, width = ToMM(pcb.ComputeBoundingBox().GetSize())
        bounding_box = pcb.ComputeBoundingBox()
        height, width = ToMM(bounding_box.GetSize())

        margin = 0 # margin around bounding boxes (currently 0)
        assert width == pytest.approx((30-10) + 0.5 + margin, 2)
        assert height == pytest.approx((20-10) + 0.5 + margin, 2)

    def test_pcb_get_pad(self):
        pcb = BOARD()
        module = FOOTPRINT(pcb)
        pcb.Add(module)
        pad = PAD(module)
        module.Add(pad)

        pad.SetShape(PAD_SHAPE_OVAL)
        pad.SetSize(VECTOR2I_MM(2.0, 3.0))
        pad.SetPosition(VECTOR2I_MM(0,0))

        # easy case
        p1 = pcb.GetPad(VECTOR2I_MM(0,0))

        # top side
        p2 = pcb.GetPad(VECTOR2I_MM(0.9,0.0))

        # bottom side
        p3 = pcb.GetPad(VECTOR2I_MM(0,1.4))

        # TODO: get pad == p1 evaluated as true instead
        #       of relying in the internal C++ object pointer
        assert pad.this == p1.this
        assert pad.this == p2.this
        assert pad.this == p3.this

    def test_pcb_save_and_load(self):
        pcb = BOARD()
        pcb.GetTitleBlock().SetTitle(self.TITLE)
        pcb.GetTitleBlock().SetComment(0,self.COMMENT1)
        result = SaveBoard(self.FILENAME,pcb)
        assert result

        pcb2 = LoadBoard(self.FILENAME)
        assert pcb2 is not None

        tb = pcb2.GetTitleBlock()
        assert tb.GetTitle() == self.TITLE
        assert tb.GetComment(0) == self.COMMENT1

        os.remove(self.FILENAME)

    def test_pcb_layer_name_set_get(self):
        pcb = BOARD()
        pcb.SetLayerName(31, BACK_COPPER)
        assert pcb.GetLayerName(31) == BACK_COPPER

    def test_pcb_layer_name_set_get(self):
        pcb = BOARD()
        pcb.SetLayerName(31, BACK_COPPER)
        assert pcb.GetLayerName(31) == BACK_COPPER

    def test_pcb_layer_id_get(self):
        pcb = BOARD()
        b_cu_id = pcb.GetLayerID(B_CU)
        pcb.SetLayerName(b_cu_id, NEW_NAME)

        # ensure we can get the ID for the new name
        assert pcb.GetLayerID(NEW_NAME) == b_cu_id

        # ensure we can get to the ID via the STD name too
        assert pcb.GetLayerID(B_CU) == b_cu_id

    def test_footprint_properties(self):
        pcb = LoadBoard("../data/pcbnew/custom_fields.kicad_pcb")
        footprint = pcb.FindFootprintByReference('J1')
        expected_fields = {
            'myfield': 'myvalue'
        }
        assert footprint.GetSheetfile() == 'custom_fields.kicad_sch'
        assert footprint.GetSheetname() == ''
        assert footprint.GetFieldText('myfield') == 'myvalue'
        assert footprint.HasField('myfield')
        assert not footprint.HasField('abcd')
        footprint.SetField('abcd', 'efgh')
        assert footprint.HasField('abcd')
        assert footprint.GetFieldText('abcd') == 'efgh'
